@@ -69,6 +69,7 @@ static gboolean finished_miners;
 
 static gint verbosity = -1;
 static gint initial_sleep = -1;
+static gboolean no_daemon;
 static gchar *eligible;
 static gboolean version;
 
@@ -83,6 +84,10 @@ static GOptionEntry entries[] = {
 	  N_("Initial sleep time in seconds, "
 	     "0->1000 (default=15)"),
 	  NULL },
+	{ "no-daemon", 'n', 0,
+	  G_OPTION_ARG_NONE, &no_daemon,
+	  N_("Runs until all configured locations are indexed and then exits"),
+	  NULL },
 	{ "eligible", 'e', 0,
 	  G_OPTION_ARG_FILENAME, &eligible,
 	  N_("Checks if FILE is eligible for being mined based on configuration"),
@@ -250,6 +255,13 @@ miner_handle_next (void)
 
 		g_message ("All miners are now finished");
 
+		/* We're not sticking around for file updates, so stop
+		 * the mainloop and exit.
+		 */
+		if (no_daemon) {
+			g_main_loop_quit (main_loop);
+		}
+
 		return;
 	}
 
@@ -259,6 +271,38 @@ miner_handle_next (void)
 	}
 }
 
+static gboolean
+miner_handle_first_cb (gpointer data)
+{
+	miner_handle_next ();
+	return FALSE;
+}
+
+static void
+miner_handle_first (TrackerConfig *config)
+{
+	gint initial_sleep;
+
+	/* If requesting to run as no-daemon, start right away */
+	if (no_daemon) {
+		miner_handle_next ();
+		return;
+	}
+
+	/* If no need to initially sleep, start right away */
+	initial_sleep = tracker_config_get_initial_sleep (config);
+	if (initial_sleep <= 0) {
+		miner_handle_next ();
+		return;
+	}
+
+	g_debug ("Performing initial sleep of %d seconds",
+	         initial_sleep);
+	g_timeout_add_seconds (initial_sleep,
+	                       miner_handle_first_cb,
+	                       NULL);
+}
+
 static void
 miner_finished_cb (TrackerMinerFS *fs,
                    gdouble         seconds_elapsed,
@@ -533,6 +577,80 @@ check_eligible (void)
 	g_object_unref (file);
 }
 
+static gboolean
+store_is_available (void)
+{
+	GDBusConnection *connection;
+	GDBusProxy *proxy;
+	gchar *name_owner;
+
+	connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
+
+	if (!connection) {
+		return FALSE;
+	}
+
+	proxy = g_dbus_proxy_new_sync (connection,
+	                               G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START,
+	                               NULL,
+	                               "org.freedesktop.Tracker1",
+	                               "/org/freedesktop/Tracker1/Status",
+	                               "org.freedesktop.Tracker1.Status",
+	                               NULL, NULL);
+
+	if (!proxy) {
+		g_object_unref (connection);
+		return FALSE;
+	}
+
+	name_owner = g_dbus_proxy_get_name_owner (proxy);
+
+	g_object_unref (connection);
+	g_object_unref (proxy);
+
+	if (name_owner) {
+		g_free (name_owner);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+miner_needs_check (TrackerMiner *miner,
+		   gboolean      store_available)
+{
+	/* Reasons to not mark ourselves as cleanly shutdown include:
+	 *
+	 * 1. Still have files to process in our queues.
+	 * 2. We crash (out of our control usually anyway).
+	 * 3. At least one of the miners is PAUSED, we have
+	 *    to exclude the situations where the miner is
+	 *    exclusively paused due to the store not being
+	 *    available, but the miner is actually done.
+	 */
+	if (!tracker_miner_is_paused (miner)) {
+		if (TRACKER_IS_MINER_FS (miner) &&
+		    tracker_miner_fs_has_items_to_process (TRACKER_MINER_FS (miner))) {
+			/* There are items left to process */
+			return TRUE;
+		}
+
+		/* We consider the miner finished */
+		return FALSE;
+	} else {
+		if (store_available) {
+			/* Paused for other reasons, so probably not done */
+			return TRUE;
+		} else {
+			/* Check whether there are more pause
+			 * reasons than the store being out.
+			 */
+			return tracker_miner_get_n_pause_reasons (miner) > 1;
+		}
+	}
+}
+
 int
 main (gint argc, gchar *argv[])
 {
@@ -545,6 +663,7 @@ main (gint argc, gchar *argv[])
 	gboolean do_mtime_checking;
 	gboolean do_crawling;
 	gboolean force_mtime_checking = FALSE;
+	gboolean store_available;
 
 	g_type_init ();
 
@@ -612,6 +731,11 @@ main (gint argc, gchar *argv[])
 
 	main_loop = g_main_loop_new (NULL, FALSE);
 
+	g_message ("Checking if we're running as a daemon:");
+	g_message ("  %s %s",
+	           no_daemon ? "No" : "Yes",
+	           no_daemon ? "(forced by command line)" : "");
+
 	/* Check if we should crawl and if we should force mtime
 	 * checking based on the config.
 	 */
@@ -678,7 +802,16 @@ main (gint argc, gchar *argv[])
 		return EXIT_FAILURE;
 	}
 	tracker_miner_fs_set_initial_crawling (TRACKER_MINER_FS (miner_applications), do_crawling);
-	tracker_miner_fs_set_mtime_checking (TRACKER_MINER_FS (miner_applications), do_mtime_checking);
+
+	/* If a locale change was detected, always do mtime checks */
+	if (tracker_miner_applications_detect_locale_changed (miner_applications)) {
+		if (!do_mtime_checking)
+			g_debug ("Forcing mtime check in applications miner as locale change was detected");
+		tracker_miner_fs_set_mtime_checking (TRACKER_MINER_FS (miner_applications), TRUE);
+	} else {
+		tracker_miner_fs_set_mtime_checking (TRACKER_MINER_FS (miner_applications), do_mtime_checking);
+	}
+
 	g_signal_connect (miner_applications, "finished",
 	                  G_CALLBACK (miner_finished_cb),
 	                  NULL);
@@ -689,22 +822,17 @@ main (gint argc, gchar *argv[])
 
 	tracker_thumbnailer_init ();
 
-	miner_handle_next ();
+	miner_handle_first (config);
 
 	/* Go, go, go! */
 	g_main_loop_run (main_loop);
 
 	g_message ("Shutdown started");
 
-	/* Reasons to not mark ourselves as cleanly shutdown include:
-	 *
-	 * 1. Still have files to process in our queues.
-	 * 2. We crash (out of our control usually anyway).
-	 * 3. At least one of the miners is PAUSED.
-	 */
-	if (!tracker_miner_fs_has_items_to_process (TRACKER_MINER_FS (miner_files)) &&
-	    !tracker_miner_is_paused (miner_applications) &&
-	    !tracker_miner_is_paused (miner_files)) {
+	store_available = store_is_available ();
+
+	if (!miner_needs_check (miner_files, store_available) &&
+	    !miner_needs_check (miner_applications, store_available)) {
 		tracker_db_manager_set_need_mtime_check (FALSE);
 	}
 
