@@ -229,7 +229,9 @@ test_ontology_change (void)
 		g_chmod (ontology_file, 0666);
 
 		tracker_data_manager_init (0, (const gchar **) test_schemas,
-		                           NULL, FALSE, 100, 100, NULL, NULL, NULL);
+		                           NULL, FALSE, 100, 100, NULL, NULL, NULL, &error);
+
+		g_assert_no_error (error);
 
 		if (g_file_get_contents (update, &queries, NULL, NULL)) {
 			gchar *query = strtok (queries, "\n");
@@ -270,7 +272,9 @@ test_ontology_change (void)
 	delete_db (FALSE);
 
 	tracker_data_manager_init (0, (const gchar **) test_schemas,
-	                           NULL, TRUE, 100, 100, NULL, NULL, NULL);
+	                           NULL, TRUE, 100, 100, NULL, NULL, NULL, &error);
+
+	g_assert_no_error (error);
 
 	for (i = 0; change_tests[i].test_name != NULL; i++) {
 		gchar *query_filename;
